﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;

namespace SimpleWebSocket
{
    public class WebSocketServerA
    {
        // protected readonly int port;
        private TcpListener currentListener = null;
        private int port;

        public WebSocketServerA(int port)
        {
            this.port = port;
            if (this.port > -1)
            {
                listen();
            }

        }

        public void listen()
        {
            currentListener = new TcpListener(IPAddress.Any, port);
            currentListener.Start();
            while (true)
            {
                using (TcpClient client = currentListener.AcceptTcpClient())
                using (NetworkStream stream = client.GetStream())
                {
                    var headers = new Dictionary<string, string>();
                    string line = string.Empty;
                    while ((line = ReadLine(stream)) != string.Empty)
                    {
                        var tokens = line.Split(new char[] { ':' }, 2);
                        if (!string.IsNullOrWhiteSpace(line) && tokens.Length > 1)
                        {
                            headers[tokens[0]] = tokens[1].Trim();
                        }
                    }

                    string secKey = headers["Sec-WebSocket-Key"];
                    string responseSecKey = ComputeWebSocketHandshakeSecurityHash09(secKey);

                    var response =
                        "HTTP/1.1 101 WebSocket Protocol Handshake" + Environment.NewLine +
                        "Upgrade: WebSocket" + Environment.NewLine +
                        "Connection: Upgrade" + Environment.NewLine +
                        "Sec-WebSocket-Origin: " + headers["Origin"] + Environment.NewLine +
                        "Sec-WebSocket-Location: ws://localhost:"+ port +"/websession" + Environment.NewLine +
                        "Sec-WebSocket-Accept: " + responseSecKey + Environment.NewLine +
                        Environment.NewLine;

                    var bufferedResponse = Encoding.UTF8.GetBytes(response);
                    stream.Write(bufferedResponse, 0, bufferedResponse.Length);

                    List<byte> lb = new List<byte>();
                    // see http://tools.ietf.org/html/draft-ietf-hybi-thewebsocketprotocol-17 
                    //     page 30 for this:
                    // 0                   1                   2                   3
                    // 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
                    // +-+-+-+-+-------+-+-------------+-------------------------------+
                    // |F|R|R|R| opcode|M| Payload len |    Extended payload length    |
                    // |I|S|S|S|  (4)  |A|     (7)     |             (16/64)           |
                    // |N|V|V|V|       |S|             |   (if payload len==126/127)   |
                    // | |1|2|3|       |K|             |                               |
                    // +-+-+-+-+-------+-+-------------+ - - - - - - - - - - - - - - - +
                    // |     Extended payload length continued, if payload len == 127  |
                    // + - - - - - - - - - - - - - - - +-------------------------------+
                    // |                               |Masking-key, if MASK set to 1  |
                    // +-------------------------------+-------------------------------+
                    // | Masking-key (continued)       |          Payload Data         |
                    // +-------------------------------- - - - - - - - - - - - - - - - +
                    // :                     Payload Data continued ...                :
                    // + - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - +
                    // |                     Payload Data continued ...                |
                    // +---------------------------------------------------------------+
                    //
                    //0x81 = 10000001 which says according to the table above:
                    //       1        it's the final message 
                    //        000     RSV1-3 are extensions which must be negotiated first
                    //           0001 opcode %x1 denotes a text frame
                    lb.Add(0x81);
                    //0x04 = 00001000
                    //       0        No mask
                    //        0001000 Rest of the 7 bytes left is the length of the payload.
                    lb.Add(0x04);
                    // add the payload
                    lb.AddRange(Encoding.UTF8.GetBytes("Test"));
                    //write it!
                    stream.Write(lb.ToArray(), 0, 6);                
                }
            }
        }

        public String ComputeWebSocketHandshakeSecurityHash09(String secWebSocketKey)
        {
            const String MagicKEY = "258EAFA5-E914-47DA-95CA-C5AB0DC85B11";
            String secWebSocketAccept = String.Empty;

            // 1. Combine the request Sec-WebSocket-Key with magic key.
            String ret = secWebSocketKey + MagicKEY;

            // 2. Compute the SHA1 hash
            SHA1 sha = new SHA1CryptoServiceProvider();
            byte[] sha1Hash = sha.ComputeHash(Encoding.UTF8.GetBytes(ret));

            // 3. Base64 encode the hash
            secWebSocketAccept = Convert.ToBase64String(sha1Hash);

            return secWebSocketAccept;
        }
        static int CountSpaces(string key)
        {
            return key.Length - key.Replace(" ", string.Empty).Length;
        }

        static string ReadLine(Stream stream)
        {
            var sb = new StringBuilder();
            var buffer = new List<byte>();
            while (true)
            {
                buffer.Add((byte)stream.ReadByte());
                var line = Encoding.ASCII.GetString(buffer.ToArray());
                if (line.EndsWith(Environment.NewLine))
                {
                    return line.Substring(0, line.Length - 2);
                }
            }
        }

        static byte[] GetBigEndianBytes(int value)
        {
            var bytes = 4;
            var buffer = new byte[bytes];
            int num = bytes - 1;
            for (int i = 0; i < bytes; i++)
            {
                buffer[num - i] = (byte)(value & 0xffL);
                value = value >> 8;
            }
            return buffer;
        }
    }
}
